# Blog

To use the blog you need to import it from `@blueberry/4finance-components/lib/blog`.

## Data
```js
import { Map } from 'immutable';

const blogPosts = Map({
  'first-post': {
    fields: {
      title: 'First Post',
      url: 'first-post',
      image: { fields: { file: { url: '//images.contentful.com/ckqhpllkz8uz/1eRjDeI2U6WGSswqG40mWk/22f56405a8ca149943936bd669d3e85a/blog.arbol.png' } } },
      publishTime: '2014-12-23',
      text: '# Lorem ipsum dolor sit amet\n consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.',
      categories: [{ title: 'General', url: 'general' }, { title: 'Supercategory', url: 'supercategory' }],
      tags: [{ title: 'General', url: 'general' }, { title: 'Money', url: 'money' }, { title: 'Article', url: 'article' }],
      metaDescription: 'description of post',
      metaKeywords: 'keywords of post',
      relatedPosts: [
        { fields: { title: 'lorem ipsum', image: { fields: { file: { fields: { url: '' } } } }, url: 'lorem-ipsum' } },
        { fields: { title: 'dolor sit amet', image: { fields: { file: { fields: { url: '' } } } }, url: 'dolor-sit-amet' } },
        { fields: { title: 'domatavus vestum', image: { fields: { file: { fields: { url: '' } } } }, url: 'domatavus-vestum' } }
      ]
    }
  }
})
```

## How to connect Data to components

```jsx
import { Menu, Homepage, ArticlePage, CategoryPage, TagPage } from `@blueberry/4finance-components/lib/blog`;

const ConnectedHomepage = () => (
  <div>
    <Menu blogPosts={blogPosts} />
    <Homepage blogPosts={blogPosts} />
  </div>
);
const ConnectedArticlePage = ({ params: { id } }) => (<ArticlePage article={blogPosts.get(id)} relatedPosts={relatedPosts} tags={tags} />);
const ConnectedCategoryPage = ({ params: { id } }) => (<CategoryPage blogPosts={blogPosts} category={id} />);
const ConnectedTagPage = ({ params: { id } }) => (<TagPage blogPosts={blogPosts} tag={id} />);
```

## Router

```jsx
<Route path="blog" component={ConnectedHomepage} />
<Route path="blog/:id" component={ConnectedArticlePage} />
<Route path="blog/tag/:id" component={ConnectedTagPage} />
<Route path="blog/category/:id" component={ConnectedCategoryPage} />
);
```
