'use strict';

exports.__esModule = true;
exports.default = undefined;

var _getOwnPropertyDescriptor = require('babel-runtime/core-js/object/get-own-property-descriptor');

var _getOwnPropertyDescriptor2 = _interopRequireDefault(_getOwnPropertyDescriptor);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _class, _desc, _value, _class2;

var _ErrorMessage = require('./ErrorMessage.react');

var _ErrorMessage2 = _interopRequireDefault(_ErrorMessage);

var _FieldWrapper = require('./wrappers/FieldWrapper.react');

var _FieldWrapper2 = _interopRequireDefault(_FieldWrapper);

var _Icon = require('../layout/Icon.react');

var _Icon2 = _interopRequireDefault(_Icon);

var _Label = require('./Label.react');

var _Label2 = _interopRequireDefault(_Label);

var _radium = require('radium');

var _radium2 = _interopRequireDefault(_radium);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _StyledComponent2 = require('../StyledComponent.react');

var _StyledComponent3 = _interopRequireDefault(_StyledComponent2);

var _coreDecorators = require('core-decorators');

var _immutable = require('immutable');

var _WarningMessage = require('./WarningMessage.react');

var _WarningMessage2 = _interopRequireDefault(_WarningMessage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

/**
* DateField component composed from 3 selects.
*/

var DateField = (0, _radium2.default)(_class = (_class2 = function (_StyledComponent) {
  (0, _inherits3.default)(DateField, _StyledComponent);

  function DateField() {
    (0, _classCallCheck3.default)(this, DateField);
    return (0, _possibleConstructorReturn3.default)(this, _StyledComponent.apply(this, arguments));
  }

  DateField.prototype.getDays = function getDays() {
    var _props = this.props,
        delimiter = _props.delimiter,
        placeholder = _props.placeholder;

    var value = this.props.value || placeholder.join(delimiter);

    var _value$split = value.split(delimiter),
        year = _value$split[0],
        month = _value$split[1];

    var range = this.getDaysRange(month, year);
    var days = (0, _immutable.Range)(1, range).map(function (day) {
      var dayValue = day < 10 ? '0' + day : day.toString();
      return { value: dayValue, text: dayValue };
    });

    return days;
  };

  /**
  * @return {String} - Date in ISO format - e.g. 1990-03-30.
  * Important props:
  * @param {Array} arrayOfMonths - Array of strings for months (if not provided, '01', '02' etc. will be used as defaults).
  * @param {Array} placeholder - Placeholders for selects in ISO format (default: ['YYYY', 'MM', 'DD']).
  * @param {String} delimiter - Separator of selects' values in the returning string (default: '-').
  */

  DateField.prototype.getDaysRange = function getDaysRange(month, year) {
    if (!month || !year) return 32;
    if (['04', '06', '09', '11'].indexOf(month) !== -1) return 31;
    if (this.isLeapYear(year) && month === '02') return 30;
    if (month === '02') return 29;

    return 32;
  };

  DateField.prototype.getMonths = function getMonths() {
    var arrayOfMonths = this.props.arrayOfMonths;

    var months = (0, _immutable.Range)(1, 13).map(function (month, index) {
      var monthValue = month < 10 ? '0' + month : month.toString();
      return { value: monthValue, text: arrayOfMonths ? arrayOfMonths[index] : monthValue };
    });
    return months;
  };

  DateField.prototype.getYears = function getYears() {
    var _props2 = this.props,
        maximumAge = _props2.maximumAge,
        minimumAge = _props2.minimumAge;

    var firstAllowedYear = new Date().getFullYear() - minimumAge;
    var lastAllowedYear = new Date().getFullYear() - maximumAge;
    var years = (0, _immutable.Range)(firstAllowedYear, lastAllowedYear - 1, -1).map(function (year) {
      var yearValue = year.toString();
      return { value: yearValue, text: yearValue };
    });

    return years;
  };

  DateField.prototype.isValidDay = function isValidDay(day, month, year) {
    var range = this.getDaysRange(month, year) - 1;
    return range >= day;
  };

  DateField.prototype.getTextValue = function getTextValue() {
    var delimiter = this.props.delimiter;
    var day = this.refs.select_0.value;
    var month = this.refs.select_1.value;
    var year = this.refs.select_2.value;


    if (this.isValidDay(day, month, year)) {
      return [year, month, day].join(delimiter);
    }

    return [year, month, '01'].join(delimiter);
  };

  DateField.prototype.getSelectValue = function getSelectValue(key) {
    var _props3 = this.props,
        delimiter = _props3.delimiter,
        placeholder = _props3.placeholder;

    var value = this.props.value || placeholder.join(delimiter);
    var normalizedKey = 2 - key; // because of ISO date format YYYY-MM-DD
    var parts = value.split(delimiter);

    if (value === undefined || value === null || parts[normalizedKey] === undefined) return '';

    return parts[normalizedKey];
  };

  DateField.prototype.isLeapYear = function isLeapYear(year) {
    if (year % 4 !== 0) return false;else if (year % 100 !== 0) return true;else if (year % 400 !== 0) return false;
    return true;
  };

  DateField.prototype.getEvent = function getEvent() {
    var name = this.props.name;


    return {
      name: name,
      value: this.getTextValue(),
      type: 'DateField'
    };
  };

  DateField.prototype.handleInputChange = function handleInputChange() {
    var onChange = this.props.onChange;


    onChange(this.getEvent());
  };

  DateField.prototype.handleInputOnBlur = function handleInputOnBlur(_ref) {
    var target = _ref.target;
    var onBlur = this.props.onBlur;


    if (target === this.refs.select_2) if (onBlur) onBlur(this.getEvent());
  };

  DateField.prototype.renderPlaceholder = function renderPlaceholder(key) {
    var placeholder = this.props.placeholder;

    var normalizedKey = 2 - key; // because of ISO date format YYYY-MM-DD

    if (!placeholder || !placeholder[normalizedKey]) return null;

    return _react2.default.createElement(
      'option',
      { disabled: true },
      placeholder[normalizedKey]
    );
  };

  DateField.prototype.renderSelect = function renderSelect(options, key) {
    var _props4 = this.props,
        name = _props4.name,
        error = _props4.error;

    var style = this.getStyle();

    return _react2.default.createElement(
      'div',
      {
        key: key,
        style: [style.selectWrapper, key === 2 && style.selectWrapper.year]
      },
      _react2.default.createElement(
        'select',
        {
          name: name + '_' + key,
          onBlur: this.handleInputOnBlur.bind(this),
          onChange: this.handleInputChange.bind(this),
          ref: 'select_' + key,
          style: [style.select, error && style.errorSelect],
          value: this.getSelectValue(key)
        },
        this.renderPlaceholder(key),
        options.map(function (option) {
          return _react2.default.createElement(
            'option',
            { key: option.value, value: option.value },
            option.text
          );
        })
      ),
      _react2.default.createElement(_Icon2.default, {
        color: style.icon.color,
        kind: 'chevron',
        size: 12,
        style: style.icon.svg,
        wrapperStyle: style.icon
      })
    );
  };

  DateField.prototype.render = function render() {
    var _this2 = this;

    var _props5 = this.props,
        label = _props5.label,
        name = _props5.name,
        error = _props5.error,
        warning = _props5.warning;

    var selects = [this.getDays(), this.getMonths(), this.getYears()];

    return _react2.default.createElement(
      _FieldWrapper2.default,
      { name: name + 'DateField', error: !!error, warning: !!warning },
      _react2.default.createElement(
        _Label2.default,
        { name: name },
        label
      ),
      selects.map(function (options, index) {
        return _this2.renderSelect(options, index);
      }),
      !!error && _react2.default.createElement(
        _ErrorMessage2.default,
        null,
        error
      ) || !!warning && _react2.default.createElement(
        _WarningMessage2.default,
        null,
        warning
      )
    );
  };

  return DateField;
}(_StyledComponent3.default), (_applyDecoratedDescriptor(_class2.prototype, 'renderSelect', [_coreDecorators.autobind], (0, _getOwnPropertyDescriptor2.default)(_class2.prototype, 'renderSelect'), _class2.prototype)), _class2)) || _class;

DateField.styleRoot = 'DateField';
DateField.propTypes = {
  arrayOfMonths: _react.PropTypes.arrayOf(_react.PropTypes.string),
  delimiter: _react.PropTypes.string,
  error: _react.PropTypes.string,
  label: _react.PropTypes.string.isRequired,
  maximumAge: _react.PropTypes.number,
  minimumAge: _react.PropTypes.number,
  name: _react.PropTypes.string.isRequired,
  onBlur: _react.PropTypes.func,
  onChange: _react.PropTypes.func.isRequired,
  placeholder: _react.PropTypes.array,
  value: _react.PropTypes.string,
  warning: _react.PropTypes.string
};
DateField.defaultProps = {
  delimiter: '-',
  maximumAge: 90,
  minimumAge: 19,
  placeholder: ['YYYY', 'MM', 'DD']
};

DateField.style = function (_ref2) {
  var colors = _ref2.colors,
      sizes = _ref2.sizes;
  return {
    errorSelect: {
      borderColor: colors.error,
      transition: 'box-shadow 0.2s',
      ':focus': {
        borderColor: '#a60017',
        boxShadow: 'inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 6px #ff405a'
      }
    },

    select: {
      width: '100%',
      height: sizes.inputHeight,
      float: 'left',
      background: colors.white,
      appearance: 'none',
      padding: '4px 30px 4px 15px',
      outline: 'none',
      cursor: 'pointer',
      position: 'relative',
      color: 'hsl(0, 0%, 30%)',
      fontWeight: '300',
      borderRadius: 0,
      borderWidth: '1px',
      borderStyle: 'solid',
      borderColor: 'hsl(0, 0%, 80%)',
      WebkitAppearance: 'none',
      transition: 'border-color .2s',
      fontSize: '14px',
      ':focus': {
        borderColor: colors.primary
      },
      '@media screen and (-ms-high-contrast: active), (-ms-high-contrast: none)': {
        padding: '4px 4px 4px 20px'
      }
    },

    selectWrapper: {
      maxWidth: '120px',
      display: 'inline-block',
      marginRight: '15px',
      position: 'relative',
      year: {
        maxWidth: '120px',
        marginRight: 0
      }
    },

    icon: {
      color: colors.black,
      position: 'absolute',
      top: 0,
      right: 0,
      height: sizes.inputHeight,
      lineHeight: sizes.inputHeight,
      userSelect: 'none',
      pointerEvents: 'none',
      width: '24px',
      '@media screen and (-ms-high-contrast: active), (-ms-high-contrast: none)': {
        display: 'none'
      },
      svg: {
        position: 'relative',
        top: '2px'
      }
    }
  };
};

exports.default = DateField;
module.exports = exports['default'];