'use strict';

exports.__esModule = true;
exports.default = undefined;

var _getOwnPropertyDescriptor = require('babel-runtime/core-js/object/get-own-property-descriptor');

var _getOwnPropertyDescriptor2 = _interopRequireDefault(_getOwnPropertyDescriptor);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _class, _desc, _value, _class2;

var _ErrorMessage = require('./ErrorMessage.react');

var _ErrorMessage2 = _interopRequireDefault(_ErrorMessage);

var _FieldWrapper = require('./wrappers/FieldWrapper.react');

var _FieldWrapper2 = _interopRequireDefault(_FieldWrapper);

var _Hint = require('./Hint.react');

var _Hint2 = _interopRequireDefault(_Hint);

var _Label = require('./Label.react');

var _Label2 = _interopRequireDefault(_Label);

var _radium = require('radium');

var _radium2 = _interopRequireDefault(_radium);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _StyledComponent2 = require('../StyledComponent.react');

var _StyledComponent3 = _interopRequireDefault(_StyledComponent2);

var _WarningMessage = require('./WarningMessage.react');

var _WarningMessage2 = _interopRequireDefault(_WarningMessage);

var _coreDecorators = require('core-decorators');

var _immutable = require('immutable');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

var KEYS = {
  BACKSPACE: 8,
  Z: 90,
  ZERO: 48,
  KEYPAD_ZERO: 96,
  KEYPAD_NINE: 105
};

var MultiField = (0, _radium2.default)(_class = (_class2 = function (_StyledComponent) {
  (0, _inherits3.default)(MultiField, _StyledComponent);

  function MultiField() {
    (0, _classCallCheck3.default)(this, MultiField);
    return (0, _possibleConstructorReturn3.default)(this, _StyledComponent.apply(this, arguments));
  }

  MultiField.prototype.getTextValue = function getTextValue() {
    var _this2 = this;

    var _props = this.props,
        delimiter = _props.delimiter,
        fields = _props.fields;


    return (0, _immutable.Range)(0, fields.length).map(function (i) {
      var field = _this2.refs['field_' + i];
      return (field.value || '').toString();
    }).join(delimiter);
  };

  MultiField.prototype.getInputValue = function getInputValue(key) {
    var _props2 = this.props,
        delimiter = _props2.delimiter,
        fields = _props2.fields,
        value = _props2.value;


    if (value === undefined || value === null) return null;

    var parts = value.split(delimiter);

    if (parts[key] === undefined) return null;

    return parts[key].substr(0, fields[key].length);
  };

  MultiField.prototype.getEvent = function getEvent() {
    var name = this.props.name;


    return {
      name: name,
      value: this.getTextValue(),
      type: 'MultiField'
    };
  };

  // with the type="number" we have to check length of the inputed value
  // manually, because maxLength doesn't work for type="number"


  MultiField.prototype.ensureInputMaxLength = function ensureInputMaxLength(element) {
    var maxLength = element.maxLength,
        value = element.value;


    if (value.length > maxLength) {
      element.value = value.substr(0, maxLength); // eslint-disable-line no-param-reassign
    }
  };

  MultiField.prototype.handleInputChange = function handleInputChange(_ref) {
    var target = _ref.target;
    var onChange = this.props.onChange;


    this.ensureInputMaxLength(target);

    onChange(this.getEvent());
  };

  MultiField.prototype.handleInputKeyUp = function handleInputKeyUp(_ref2) {
    var keyCode = _ref2.keyCode,
        _ref2$target = _ref2.target,
        maxLength = _ref2$target.maxLength,
        name = _ref2$target.name,
        value = _ref2$target.value;

    var currentFieldIndex = parseInt(name.split('_')[1], 10);
    var isDeletingChar = keyCode === KEYS.BACKSPACE;
    var isLetterOrNumber = keyCode >= KEYS.ZERO && keyCode <= KEYS.Z || keyCode >= KEYS.KEYPAD_ZERO && keyCode <= KEYS.KEYPAD_NINE;
    var isEmptyValue = value === '';
    var isInFirstField = currentFieldIndex === 0;
    var isInLastField = currentFieldIndex === this.props.fields.length - 1;
    var hasReachedCharLimit = value.length === maxLength;

    if (!isDeletingChar && !isLetterOrNumber) return;

    if (isDeletingChar && isEmptyValue && !isInFirstField) {
      this.jumpToPreviousField(currentFieldIndex);
    } else if (hasReachedCharLimit && !isInLastField) {
      this.jumpToNextBox(currentFieldIndex);
    }
  };

  MultiField.prototype.handleInputOnBlur = function handleInputOnBlur(_ref3) {
    var target = _ref3.target;
    var _props3 = this.props,
        onBlur = _props3.onBlur,
        fields = _props3.fields;

    var shouldBlur = target === this.refs['field_' + (fields.length - 1)];

    if (!onBlur) {
      return;
    }

    if (shouldBlur) {
      onBlur(this.getEvent());
    }
  };

  MultiField.prototype.jumpToField = function jumpToField(fieldIndex) {
    this.refs['field_' + fieldIndex].focus();
  };

  MultiField.prototype.jumpToPreviousField = function jumpToPreviousField(currentFieldIndex) {
    this.jumpToField(currentFieldIndex - 1);
  };

  MultiField.prototype.jumpToNextBox = function jumpToNextBox(currentFieldIndex) {
    this.jumpToField(currentFieldIndex + 1);
  };

  MultiField.prototype.renderHint = function renderHint() {
    var hint = this.props.hint;


    return hint ? _react2.default.createElement(
      _Hint2.default,
      null,
      hint
    ) : null;
  };

  MultiField.prototype.getLabel = function getLabel(labelString) {
    var name = this.props.name;


    return labelString ? _react2.default.createElement(
      _Label2.default,
      { name: name },
      labelString
    ) : null;
  };

  MultiField.prototype.renderLabelByIndex = function renderLabelByIndex(inputIndex) {
    var label = this.props.label;

    var isMultiLabel = Array.isArray(label);
    var labelString = label && label[inputIndex];

    return isMultiLabel ? this.getLabel(labelString) : null;
  };

  MultiField.prototype.renderWrapperLabel = function renderWrapperLabel() {
    var label = this.props.label;

    var isMultiLabel = Array.isArray(label);

    return !isMultiLabel ? this.getLabel(label) : null;
  };

  MultiField.prototype.renderError = function renderError() {
    var error = this.props.error;


    return error ? _react2.default.createElement(
      _ErrorMessage2.default,
      null,
      error
    ) : null;
  };

  MultiField.prototype.renderWarning = function renderWarning() {
    var warning = this.props.warning;


    return warning ? _react2.default.createElement(
      _WarningMessage2.default,
      null,
      warning
    ) : null;
  };

  MultiField.prototype.renderInput = function renderInput(field, key) {
    var _props4 = this.props,
        delimiter = _props4.delimiter,
        fields = _props4.fields,
        name = _props4.name,
        error = _props4.error;
    var length = field.length,
        type = field.type,
        placeholder = field.placeholder;

    var style = this.getStyle();
    var lengthStyle = { width: length + 'em' };

    return _react2.default.createElement(
      'span',
      { key: key, style: style.fieldGroup },
      this.renderLabelByIndex(key),
      _react2.default.createElement('input', {
        maxLength: length,
        name: name + '_' + key,
        onBlur: this.handleInputOnBlur,
        onChange: this.handleInputChange,
        onKeyUp: this.handleInputKeyUp,
        ref: 'field_' + key,
        placeholder: placeholder,
        size: length < 10 ? length : length + 4,
        style: [style.input, error && style.errorInput, lengthStyle],
        type: type,
        value: this.getInputValue(key)
      }),
      key !== fields.length - 1 && _react2.default.createElement(
        'span',
        { className: 'delimiter', style: style.delimiter },
        delimiter
      )
    );
  };

  MultiField.prototype.render = function render() {
    var _props5 = this.props,
        fields = _props5.fields,
        name = _props5.name,
        error = _props5.error,
        warning = _props5.warning;


    return _react2.default.createElement(
      _FieldWrapper2.default,
      { name: name + 'MultiField', error: !!error, warning: !!warning },
      this.renderWrapperLabel(),
      fields.map(this.renderInput, this),
      this.renderHint(),
      this.renderError() || this.renderWarning()
    );
  };

  return MultiField;
}(_StyledComponent3.default), (_applyDecoratedDescriptor(_class2.prototype, 'handleInputChange', [_coreDecorators.autobind], (0, _getOwnPropertyDescriptor2.default)(_class2.prototype, 'handleInputChange'), _class2.prototype), _applyDecoratedDescriptor(_class2.prototype, 'handleInputKeyUp', [_coreDecorators.autobind], (0, _getOwnPropertyDescriptor2.default)(_class2.prototype, 'handleInputKeyUp'), _class2.prototype), _applyDecoratedDescriptor(_class2.prototype, 'handleInputOnBlur', [_coreDecorators.autobind], (0, _getOwnPropertyDescriptor2.default)(_class2.prototype, 'handleInputOnBlur'), _class2.prototype), _applyDecoratedDescriptor(_class2.prototype, 'renderInput', [_coreDecorators.autobind], (0, _getOwnPropertyDescriptor2.default)(_class2.prototype, 'renderInput'), _class2.prototype)), _class2)) || _class;

MultiField.styleRoot = 'MultiField';
MultiField.propTypes = {
  delimiter: _react.PropTypes.string,
  error: _react.PropTypes.string,
  fields: _react.PropTypes.arrayOf(_react.PropTypes.shape({
    length: _react.PropTypes.number.isRequired,
    type: _react.PropTypes.oneOf(['text', 'number']),
    placeholder: _react.PropTypes.string
  })),
  hint: _react.PropTypes.string,
  label: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.arrayOf(_react.PropTypes.string)]).isRequired,
  name: _react.PropTypes.string.isRequired,
  onBlur: _react.PropTypes.func,
  onChange: _react.PropTypes.func.isRequired,
  value: _react.PropTypes.string,
  warning: _react.PropTypes.string
};
MultiField.defaultProps = {
  delimiter: '-',
  fields: [{ length: 1, type: 'text' }, { length: 1, type: 'text' }, { length: 1, type: 'text' }, { length: 1, type: 'text' }]
};

MultiField.style = function (_ref4) {
  var colors = _ref4.colors;
  return {
    fieldGroup: {
      display: 'inline-block'
    },

    errorInput: {
      borderColor: colors.error,
      transition: 'box-shadow 0.2s',
      ':focus': {
        borderColor: '#a60017',
        boxShadow: 'inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 6px #ff405a'
      }
    },
    input: {
      borderColor: colors.inputBorder,
      display: 'inline-block',
      lineHeight: '40px',
      marginRight: '0 6px',
      padding: '0px 12px',
      color: 'hsl(0, 0%, 30%)',
      fontWeight: '300',
      borderWidth: '1px',
      borderStyle: 'solid',
      outline: 'none',
      WebkitAppearance: 'none',
      transition: 'border-color .2s',
      height: '40px',
      textAlign: 'center',
      fontSize: '14px',
      ':focus': {
        borderColor: colors.primary
      }
    },
    delimiter: {
      margin: '0 10px',
      lineHeight: '40px',
      verticalAlign: 'middle'
    }
  };
};

exports.default = MultiField;
module.exports = exports['default'];