# Theme styling


Every component has possibility to setup `static style` which can be `{}` or function
which will get `globals` variables as first argument.

Component should use only styles which are defined in `static style`.

When using component, you can override style on two levels
`<ThemeProvider style={}>` or `<YourComponent style={}>`.

## Themes

### Onea
```
import onea from '@blueberry/4finance-components/lib/styles/onea'
import { ThemeProvider } from '@blueberry/4finance-components'


<ThemeProvider style={onea}>
</ThemeProvider>
```

### Vivus
```
import vivus from '@blueberry/4finance-components/lib/styles/vivus'
import { ThemeProvider } from '@blueberry/4finance-components'


<ThemeProvider style={vivus}>
</ThemeProvider>
```

## Components

`YourComponent` is accepting `{}` or function which will get `globals` variables as first argument.

`<ThemeProvider style={}>` is accepting only `{}` which is in format

```javascript
{
  globals: {
    ...
  },
  YourComponent: {}
  ...
}
```

name in theme is by default taken from name of component but you are able to
override this by setting `static styleRoot`.

```jsx
import StyledComponent from './StyledComponent.react';
import React from 'react';

class Button extends StyledComponent {

  render() {
    const style = this.getStyle();

    return (
      <button style={[this.props.disabled && style.disable]}>
        ABC
      </button>
    );
  }

  static style = (globals) => ({
    base: {
      backgroundColor: globals.colors.primary
    },
    disabled: {
      color: globals.colors.secondary
    }
  })
}


ReactDOM.renderToString(
  <ThemeProvider style={{
    globals: {
      colors: {
        primary: 'green',
        secondary: 'red'
      }
    },
    Button: {
      base: {
        width: '100%',
        padding: '50px'
      }
    }
  }}>
    <Button style={(globals) => ({
      base: {
        color: globals.colors.secondary,
        width: '20px'
      }
    })}/>
  </ThemeProvider>
)
```
